import os, sys, getopt
hel = """
usage: nought [options] [identifiers ...]
    Runs `nought`, the super customizable file cleaner/organizer/automator
    
    Options:
			-c [location], --config [location]  	specify configuration file. Default: "./nought.toml" (see -d and "Environment Variables")
			-b [location], --backup [location]		backup files to specified location. Default: false
			-d, --default		set the config file as default
			-a, --about			display version and exit
			-h, --help			display this help message
			-t, --test			test configuration without actually doing anything (enables verbose mode)
    
    Arguments:
			IDENTIFIERS:		a repeatable identifier for which groups to run
    
    Environment Variables:
			NOUGHT_CONFIG:  the path to the default configuration file
"""
def help():
	global hel
	try:
		helptext = open(os.path.dirname(os.path.abspath(__file__))+os.sep+"help.txt").read()
	except:
		helptext = hel
	print(helptext)

def main():
	options = {}
	try:
		arguments, ids = getopt.getopt(sys.argv[1:], "hdbsvmwatc:", ["help","wizard","witch","squib","muggle","default","config=",'backup =','about','test','easteregg'])
	except getopt.GetoptError as e:
		print(e)
		print('\n')
		help()
		sys.exit(0)
	if len(ids) == 0:
		ids = ["default"]
	options["verbose"] = False
	options["test"] = False
	for key,value in arguments:
		if key in ("--easteregg"):
			print("Don't be silly, there aren't any easter eggs hidden in the program... i think...")
			sys.exit()
		if key in ("--help", "-h"):
			help()
			sys.exit()
		if key in ("--muggle", "-m"):
			print("There aren't any wizards here, what are you talking about?\n**OBLIVIATE!!**")
			sys.exit()
		
		if key in ("--wizard", "-w","--witch","--squib",'-s'):
			import nought.wizard as wizard
			wizard.main()
		
		if key in ("--about", "-a"):
			try:
				print('\nConfig file at: {}'.format(os.environ["NOUGHT_CONFIG"]))
			except KeyError:
				print('Config file at: NOT DEFINED YET\nto specify a default one, change the `NOUGHT_CONFIG` env variable\nor place your file at `'+os.path.dirname(os.path.abspath(__file__))+os.sep+"conf.toml`")
			sys.exit()

		if key in ("--verbose", "-v"):
			options['verbose'] = True

		if key in ("--backup", "-b"):
			options['backup'] = True

		if key in ("--test", "-t"):
			options['test'] = True

		if key in ("--config", "-c"):
			options['config'] = value

		if key in ("--default", "-d"):
			options['def'] = True
		if key in ("--config", "-c"):
			options['config'] = value
		
	if "config" in options:
		pass
	elif "NOUGHT_CONFIG" in os.environ:
		options['config'] = os.environ["NOUGHT_CONFIG"]
	elif os.path.isfile(os.path.abspath(__file__).replace(__file__,"")+"conf.toml"):
		options['config'] = os.path.abspath(__file__).replace(__file__,"")+"conf.toml"
	elif os.name == 'nt':
		print('You are using Windows, so you have to manually set a config location with -c or --config or place your file at: '+os.path.dirname(os.path.abspath(__file__))+os.sep+"conf.toml")
		sys.exit(1)
	# print(os.abspath(__file__))
	elif os.path.isfile('nought.toml'):
		options['config'] = 'nought.toml'
	else:
		print('nought: no config file found!\n')
		help()
		sys.exit(0)
	if "def" in options and options['def']:
		print('To set a default path to config, please change the NOUGHT_CONFIG env variable like so:')
		print('\n\texport NOUGHT_CONFIG={}'.format(options['config']))
		print('\nor place your file at \n\t'+os.path.dirname(os.path.abspath(__file__))+os.sep+"conf.toml")
		sys.exit(0)
	if "verbose" in options and options["verbose"]:
		print("using configuration file: "+options['config'])
	import nought.runner as runner
	runner.main(options,ids)

if __name__=="__main__":
	main()


#don't look here... that's cheating

































































































def egg():
	egg = "i told you there weren't any easter eggs!"
	print(egg)


"""
NOUGHT: a File Organizer
by Cole Wilson
colewilson.xyz
github.com/cole-wilson/nought

"""
import os, toml, re, datetime, time, requests, sys, getopt

verb = False
tab = 0
f = ""
def shellquote(s):
	return s.replace('(',r"\(").replace(')',r"\)")#.replace(' ',r"\ ")
def e(string):
	return re.sub("_\{.*?\}",ev,str(string))
def p(string):
	global verb
	global tab
	if verb:
		print("\t"*tab+str(string))
	else:
		return
def ensure(path):
	global rule
	# print(path.split(os.sep)[:-1])
	sp = path.split(os.sep)[:-1]
	d = os.sep.join(sp)
	if not os.path.isdir(d) and d!="":
		os.system('mkdir -p ' + d)
	# print(d)
def move(rule,options):
	global f
	if not "regex" in rule:#NORMAL MOVE
		newgroup = e(rule["move_to"].replace('$bp',basepath))+os.sep+f
		ensure(newgroup)
		if not options['test']:
			p("moving file to `"+newgroup+"`")
			os.rename(f,newgroup)

	else:#REGEX MOVE
		# try:
		newgroup = e(re.sub(".*?"+rule["regex"]+".*",rule['move_to'].replace('$bp',basepath).replace('$name',f.split(os.sep)[-1]).replace('$','\\'),f))
		p("moving file to `"+newgroup+"`")
		if not options['test']:
			ensure(newgroup)
			os.rename(f,newgroup)
		# except:
			# pass
def ev(match):
	t = match.group().replace('_','')
	return str(next(iter((eval(t.replace('$bp',basepath).replace('$name','"'+f.split(os.sep)[-1]+'"'))))))
def main(options,ids):
	global f
	try:
		with open(options['config'],'r') as f:
			config = toml.loads(f.read())
	except FileNotFoundError:
		print("nought: error reading config: `"+options['config']+"` not found")
		sys.exit(0)
	except toml.decoder.TomlDecodeError as error:
		print("nought: error in reading config file `"+options['config']+"`:\n\t"+str(error))
		sys.exit(0)		
	if options["verbose"] or options["test"]:
		global verb
		verb = True
	global tab
	global basepath
	for group in config["group"]:
		if "id" not in group:
			group["id"] = ids
		if isinstance(group["id"],str):
			group["id"] = [group["id"]]
		pa = True
		print('IDs: '+str(ids))
		for x in group["id"]:
			if x in ids:
				pa = False
		if pa:
			continue
		if "base_dir" in config["general"]:
			p("Going to base directory "+config["general"]["base_dir"])
			os.chdir(config["general"]["base_dir"])
		path = group["path"]
		if isinstance(path,str):
			path = [path]
		p("Going to group with locations " + str(path) + ":")
		for path in path:
			basepath = os.path.abspath(path)
			if "backup" in options:
				ensure(options["backup"])
				os.system('cd -r {} {}'.format(path,options["backup"]))
			try:
				os.chdir(path)
			except FileNotFoundError:
				p("That didn't work! That folder doesn't exist.")
				continue
			tab += 1
			filedirs = []
			# get files and folders
			if "recursive" in group and group["recursive"]:
				for root, dirs, files in os.walk("."):
					path = root.split(os.sep)
					for file in files:
							filedirs.append(root.replace('.'+os.sep,'')+os.sep+file.replace('.'+os.sep,''))
					for dir in dirs:
							filedirs.append(root.replace('.'+os.sep,'')+os.sep+dir.replace('.'+os.sep,''))
			else:
				filedirs = os.listdir(".")
			files = []
			dirs = []
			for x in filedirs:
				if os.path.isfile(x):
					files.append(x.replace('.'+os.sep,''))
				elif os.path.isdir(x):
					dirs.append(x.replace('.'+os.sep,''))
			if ("include_dirs" in group and group["include_dirs"]):
				files = filedirs
			if ("only_dirs" in group and group["only_dirs"]):
				files = dirs
			# rules
			filesmatched = []
			for f in files:		
				if "debug" in config["general"] and config["general"]["debug"]:
					input('Press enter to continue...')
				f = f.replace('.'+os.sep,'')
				pas = False
				if f.startswith('.'):
					if "include_hidden" in group:
						if group["include_hidden"]:
							pas = False
						else:
							pas = True
					else:
						pas = True
				else:
					pas = False
				if pas:
					continue
				if "include_dirs" in group and group["include_dirs"] and len(f.split(os.sep))>1:
					continue
				# f = f.replace(' ','_')
				# f = f.replace('(','_')
				# f = f.replace(')','_')
				newgroup = f
				p("- "+f)
				nomatch = True
				if "rule" not in group:
					group["rule"] = []
				for rule in group["rule"]:
					if f in filesmatched:
						break
					match = False
					tab += 1
					a = 0
					am = 0
					matches = "matches: "
					for x in ["includes","regex","size_less_than","size_more_than","content_includes","custom","modified_before","modified_after","user","group"]:
						if x in rule:
							a += 1
					if "includes" in rule:
						for include in rule["includes"]:
							if include in f:
								am += 1
								matches += " `"+include+"`"
					if "custom" in rule:
						if eval(rule["custom"].replace('$bp',basepath).replace('$name','"'+f.split(os.sep)[-1]+'"')):
							am += 1
							matches += " `"+rule['custom']+"`"
					if "regex" in rule:
						if re.search(rule["regex"],f):
							matches += " `"+rule["regex"]+"`"
							am += 1
					if "size_less_than" in rule:
						if os.stat(f).st_size < int(e(rule["size_less_than"])):
							matches += " size<`"+str(rule["size_less_than"])+"`"
							am += 1
					if "size_more_than" in rule:
						if os.stat(f).st_size < int(e(rule["size_more_than"])):
							matches += " size>`"+str(rule["size_more_than"])+"`"
							am += 1
					if "modified_before" in rule:
						if rule["modified_before"] < 0:
							rule["modified_before"] - time.time()
						if os.stat(f).st_ctime < int(e(rule["modified_before"])):
							matches += " modified_before `"+str(rule["modified_before"])+"`"
							am += 1
					if "modified_after" in rule:
						if rule["modified_after"] < 0:
							rule["modified_after"] - time.time()
						if os.stat(f).st_ctime > int(e(rule["modified_after"])):
							matches += " modified_after `"+str(rule["modified_after"])+"`"
							am += 1
					if "user" in rule:
						if os.popen('getent passwd '+str(os.stat(f).st_uid)+' | cut -d: -f1').read().replace('\n','') == e(rule["user"]):
							matches += " owned_by_user `"+str(rule["user"])+"`"
							am += 1
					if "group" in rule:
						if os.popen('getent passwd '+str(os.stat(f).st_gid)+' | cut -d: -f1').read().replace('\n','') == e(rule["group"]):
							matches += " owned_by_group `"+str(rule["group"])+"`"
							am += 1
					if "content_includes" in rule:
						if e(rule["content_includes"]) in open(f).read():
							matches += " content_includes:`"+str(rule["content_includes"])+"`"
							am += 1
					if am == a:
						match = True
						nomatch = False
						p(matches)
					if match:
						# DELETE
						filesmatched.append(f)
						if e(rule["action"]) == "delete":
							p("deleting item")
							if not options['test']:
								os.remove(f)
						# MOVE
						elif e(rule["action"]) == "move":
							move(rule,options)
						# SCRIPT
						elif "script" in rule:
							try:
								p("ruinning script: {}".format(shellquote(rule["script"].replace('$bp',basepath).replace('$name',newgroup.split(os.sep)[-1]))))
								if not options['test']:
									os.system(shellquote(rule["script"].replace('$bp',basepath).replace('$name',newgroup.split(os.sep)[-1])))
							except:
								pass
						# LINK
						if "link_in" in rule:
							if "regex" in rule:
								linkloc = re.sub(rule["regex"],e(rule['link_in']).replace('$bp',basepath).replace('$name',f.split(os.sep)[-1]).replace('$','\\'),f)
							else:
								linkloc = e(rule['link_in'])
							p(f'linking to file from {linkloc}')
							if not options['test']:
								ensure(linkloc)
								os.system(f'ln -s {newgroup} {linkloc}')
					tab -= 1
				if nomatch:
					tab += 1
					try:
						p(f"didn't match, doing action `{group['other']['action']}`")
					except KeyError:
						pass
					tab += 1
					try:
						rule = group['other']
					except:
						rule = {"action":"none"}
					if e(rule["action"]) == "delete":
						p("deleting item")
						try:
							if not options['test']:
								os.remove(f)
						except IsADirectoryError:
							try:
								os.rmdir(f)
							except:
								pass
					# MOVE
					elif e(rule["action"]) == "move":
						move(rule,options)
					if "script" in rule:
						try:
							os.system(shellquote(rule["script"].replace('$bp',basepath).replace('$name',newgroup.split(os.sep)[-1])))
						except:
							pass
					if "link_in" in rule:
						if "regex" in rule:
							linkloc = re.sub(rule["regex"],e(rule['link_in']).replace('$bp',basepath).replace('$name',f.split(os.sep)[-1]).replace('$','\\'),f)
						else:
							linkloc = e(rule['link_in'])
						p(f'linking to file from {linkloc}')
						ensure(linkloc)
						os.system(f'ln -s {newgroup} {linkloc}')
					tab -= 2
			tab -= 1


import sys, os, toml 
def main():
	# sys.exit(0)
	print('Hello! The best way to setup nought is to go to the github page:\n\thttps://github.com/cole-wilson/nought#configuration')
	# mode = input("\nYou have several choices:\n\t\1:Create a config file.\n\t2: Edit an existing config file.\n\t(your choice must be an integer)\n>>> ")
	# try:
	# 	mode = int(mode)
	# except:
	# 	print("You have to chose an integer!")
	# if mode == 1:
	# 	fname = input('Please type the path to your pre-existing file.\n>>> ')
	# 	fname = os.path.abspath(fname)
	# 	try:
	# 		conf = toml.loads(open(fname).read())
	# 		print(conf)
	# 	except toml.decoder.TomlDecodeError as error:
	# 		print("This file is malformed:\n\t"+str(error)+"\nPlease fix!")
	# 		sys.exit(1)
	# 	except IsADirectoryError:
	# 		print('That path is a directory!')
	# 		sys.exit(1)
	# 	except FileNotFoundError:
	# 		print('That path doesn\'t exist!')
	# 		sys.exit(1)
	# 	for group in conf["group"]:
	# 		t = input("Please enter a cron style time for when I should run this command:\n>>> ")
	sys.exit(0)


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="nought", # Replace with your own username
    version="1.0.6",
		scripts=['bin/nought','bin/nt'],
		entry_points={
        'console_scripts': ['nought=nought.__main__:main'],
    },
    author="Cole Wilson",
    author_email="cole@colewilson.xyz",
    description="A super customizable file cleaner/organizer/automator",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/cole-wilson/nought",
    packages=setuptools.find_packages(),
		install_requires=[
   		'toml','requests'
		],
    classifiers=[
        "Programming Language :: Python :: 3",
    ],
    python_requires='>=3.6',
		package_data={
      "": ["*.toml","*.txt"],
    },
		license="Apache-2.0",
		keywords='nought file organizer desktop cleaner declutterer devtools tools tool',
    project_urls={
        'Documentation': 'https://github.com/cole-wilson/nought',
        'Questions': 'https://github.com/cole-wilson/nought',
        'Source': 'https://github.com/cole-wilson/nought',
        'Tracker': 'https://github.com/cole-wilson/nought/issues',
    },
)
